@@ -34,7 +34,9 @@ task_serializer = 'pickle'
 task_inherit_parent_priority = True
 task_default_priority = 0
 task_queue_max_priority = 1
-priority_steps = list(range(task_queue_max_priority + 1))
+broker_transport_options = {
+    'priority_steps': list(range(task_queue_max_priority + 1))
+}
 
 # GWCelery-specific settings.
 
@@ -98,7 +100,7 @@ superevent_default_d_t_end = 1.0
 superevent_far_threshold = 1 / 3600
 """Maximum false alarm rate to consider events superevents."""
 
-preliminary_alert_timeout = 30.
+preliminary_alert_timeout = 30.0
 """Wait this many seconds for the preferred event to stabilize before issuing a
 preliminary alert."""
 
@@ -113,6 +115,9 @@ preliminary_alert_trials_factor = dict(cbc=5.0, burst=4.0)
 factor is the number of pipelines. CBC pipelines are gstlal, pycbc, mbtaonline
 and spiir. Burst searches are cwb.allsky, cwb.bbh and cwb.imbh."""
 
+early_warning_alert_far_threshold = preliminary_alert_far_threshold['cbc']
+"""False alarm rate threshold for early warning alerts."""
+
 snews_gw_far_threshold = 1 / (3600 * 24)
 """Maximum false alarm rate for a superevent to send out a coincidence alert
 between an external SNEWS alert and the superevent."""
@@ -314,3 +319,6 @@ within time window of each other."""
 mock_events_simulate_multiple_uploads = False
 """If True, then upload each mock event several times in rapid succession with
 random jitter in order to simulate multiple pipeline uploads."""
+
+# Delete imported modules so that they do not pollute the config object
+del os, getpass
