@@ -12,17 +12,22 @@ References
 .. [LIGO] https://wiki.ligo.org/Calibration/TDCalibReview
 .. [Virgo] https://dcc.ligo.org/G1801125/
 .. [DMT] https://wiki.ligo.org/DetChar/DmtDqVector
+
 """
 import getpass
 import glob
+import io
 import json
 import socket
 import time
 
+from celery import group
 from celery.utils.log import get_task_logger
 from glue.lal import Cache
 from gwdatafind import find_urls
 from gwpy.timeseries import Bits, StateVector, TimeSeries
+from gwpy.plot import Plot
+import matplotlib.pyplot as plt
 import numpy as np
 
 from . import gracedb
@@ -35,64 +40,6 @@ __author__ = 'Geoffrey Mo <geoffrey.mo@ligo.org>'
 
 log = get_task_logger(__name__)
 
-dmt_dq_vector_bits = Bits(
-    channel='DMT-DQ_VECTOR',
-    bits={
-        1: 'NO_OMC_DCPD_ADC_OVERFLOW',
-        2: 'NO_DMT-ETMY_ESD_DAC_OVERFLOW'
-    },
-    description={
-        'NO_OMC_DCPD_ADC_OVERFLOW': 'OMC DCPC ADC not overflowing',
-        'NO_DMT-ETMY_ESD_DAC_OVERFLOW': 'ETMY ESD DAC not overflowing'
-    }
-)
-"""DMT DQ vector bits (LIGO only)."""
-
-
-ligo_state_vector_bits = Bits(
-    channel='GDS-CALIB_STATE_VECTOR',
-    bits={
-        0: 'HOFT_OK',
-        1: 'OBSERVATION_INTENT',
-        5: 'NO_STOCH_HW_INJ',
-        6: 'NO_CBC_HW_INJ',
-        7: 'NO_BURST_HW_INJ',
-        8: 'NO_DETCHAR_HW_INJ'
-    },
-    description={
-        'HOFT_OK': 'h(t) was successfully computed',
-        'OBSERVATION_INTENT': '"observation intent" button is pushed',
-        'NO_STOCH_HW_INJ': 'No stochastic HW injection',
-        'NO_CBC_HW_INJ': 'No CBC HW injection',
-        'NO_BURST_HW_INJ': 'No burst HW injection',
-        'NO_DETCHAR_HW_INJ': 'No HW injections for detector characterization'
-    }
-)
-"""State vector bitfield definitions for LIGO."""
-
-virgo_state_vector_bits = Bits(
-    channel='DQ_ANALYSIS_STATE_VECTOR',
-    bits={
-        0: 'HOFT_OK',
-        1: 'OBSERVATION_INTENT',
-        5: 'NO_STOCH_HW_INJ',
-        6: 'NO_CBC_HW_INJ',
-        7: 'NO_BURST_HW_INJ',
-        8: 'NO_DETCHAR_HW_INJ',
-        10: 'GOOD_DATA_QUALITY_CAT1'
-    },
-    description={
-        'HOFT_OK': 'h(t) was successfully computed',
-        'OBSERVATION_INTENT': '"observation intent" button is pushed',
-        'NO_STOCH_HW_INJ': 'No stochastic HW injection',
-        'NO_CBC_HW_INJ': 'No CBC HW injection',
-        'NO_BURST_HW_INJ': 'No burst HW injection',
-        'NO_DETCHAR_HW_INJ': 'No HW injections for detector characterization',
-        'GOOD_DATA_QUALITY_CAT1': 'Good data quality (CAT1 type)'
-    }
-)
-"""State vector bitfield definitions for Virgo."""
-
 
 def create_cache(ifo, start, end):
     """Find .gwf files and create cache. Will first look in the llhoft, and
@@ -118,32 +65,113 @@ def create_cache(ifo, start, end):
      ...
       <glue.lal.CacheEntry at 0x7fbae6b15080>,
       <glue.lal.CacheEntry at 0x7fbae6b15828>]
+
     """
     pattern = app.conf['llhoft_glob'].format(detector=ifo)
     filenames = glob.glob(pattern)
     cache = Cache.from_urls(filenames)
+
     try:
         cache_starttime = int(
             list(cache.to_segmentlistdict().values())[0][0][0])
     except IndexError:
         log.exception('Files do not exist in llhoft_glob')
         return cache  # returns empty cache
-    if start < cache_starttime:  # required data has left llhoft
-        high_latency = app.conf['high_latency_frame_types'][ifo]
-        urls = find_urls(ifo[0], high_latency, start, end)
-        if len(urls) != 0:
-            return Cache.from_urls(urls)
-        else:  # required data not in high latency frames
-            low_latency = app.conf['low_latency_frame_types'][ifo]
-            urls = find_urls(ifo[0], low_latency, start, end)
-            if len(urls) != 0:
-                return Cache.from_urls(urls)
-            else:  # required data not in low latency frames
-                error_msg = "This data cannot be found, or does not exist."
-                log.exception(error_msg)
-    else:
+
+    if start >= cache_starttime:  # required data is in llhoft
         return cache
 
+    # otherwise, required data has left llhoft
+    high_latency = app.conf['high_latency_frame_types'][ifo]
+    urls = find_urls(ifo[0], high_latency, start, end)
+    if urls:
+        return Cache.from_urls(urls)
+
+    # required data not in high latency frames
+    low_latency = app.conf['low_latency_frame_types'][ifo]
+    urls = find_urls(ifo[0], low_latency, start, end)
+    if not urls:  # required data not in low latency frames
+        log.error('This data cannot be found, or does not exist.')
+
+    return Cache.from_urls(urls)
+
+
+@app.task(shared=False)
+def make_omegascan(ifo, t0, durs):
+    """Helper function to create a single omegascan image, with
+    multiple durations.
+
+    Parameters
+    ----------
+    ifo : str
+        'H1', 'L1', or 'V1'
+    t0 : int or float
+        Central time of the omegascan.
+    durs : list of floats/ints
+        List of three durations which will be scanned symmetrically about t0.
+        Example: [0.5, 2, 10]
+
+    Returns
+    -------
+    bytes or None
+        bytes of png of the omegascan, or None if no omegascan created.
+
+    """
+    # Collect data
+    longest = max(durs)
+    long_start, long_end = t0 - longest, t0 + longest
+    cache = create_cache(ifo, long_start, long_end)
+    strain_name = app.conf['strain_channel_names'][ifo]
+    try:
+        ts = TimeSeries.read(cache, strain_name,
+                             start=long_start, end=long_end).astype('float64')
+        # Do q_transforms for the different durations
+        qgrams = [ts.q_transform(
+            frange=(20, 4096), gps=t0, outseg=(t0 - dur, t0 + dur), logf=True)
+            for dur in durs]
+    except (IndexError, FloatingPointError, ValueError):
+        # data from cache can't be properly read, or data is weird
+        fig = plt.figure()
+        plt.axis("off")
+        plt.text(0.1, 0.45, f"Failed to create {ifo} omegascan", fontsize=17)
+    else:
+        fig = Plot(*qgrams,
+                   figsize=(10 * len(durs), 5),
+                   geometry=(1, len(durs)),
+                   yscale='log',
+                   method='pcolormesh')
+        for ax in fig.axes:
+            fig.colorbar(ax=ax, label='Normalized energy', clim=(0, 30))
+            ax.set_epoch(t0)
+        fig.suptitle(f'Omegascans of {strain_name} at {t0}', fontweight="bold")
+
+    outfile = io.BytesIO()
+    fig.savefig(outfile, format='png', dpi=300)
+    return outfile.getvalue()
+
+
+@app.task(shared=False)
+def omegascan(t0, graceid):
+    """Create omegascan for a certain event.
+
+    Parameters
+    ----------
+    t0 : float
+        Central event time.
+    graceid : str
+        GraceDB ID to which to upload the omegascan.
+
+    """
+    durs = app.conf['omegascan_durations']
+
+    group(
+        make_omegascan.s(ifo, t0, durs)
+        |
+        gracedb.upload.s(f"{ifo}_omegascan.png", graceid,
+                         f"{ifo} omegascan", tags=['data_quality'])
+        for ifo in ['H1', 'L1', 'V1']
+    ).delay()
+
 
 def generate_table(title, high_bit_list, low_bit_list, unknown_bit_list):
     """Make a nice table which shows the status of the bits checked.
@@ -163,6 +191,7 @@ def generate_table(title, high_bit_list, low_bit_list, unknown_bit_list):
     -------
     str
         HTML string of the table.
+
     """
     template = env.get_template('vector_table.jinja2')
     return template.render(title=title, high_bit_list=high_bit_list,
@@ -237,18 +266,21 @@ def check_idq(cache, channel, start, end):
     >>> check_idq(cache, 'H1:IDQ-PGLITCH-OVL-100-1000',
                   1216496260, 1216496262)
     ('H1:IDQ-PGLITCH-OVL-100-1000', 0.87)
+
     """
-    try:
-        idq_prob = TimeSeries.read(
-            cache, channel, start=start, end=end)
-    except (IndexError, RuntimeError):
-        # FIXME: figure out how to get access to low-latency frames outside
-        # of the cluster. Until we figure that out, actual I/O errors have
-        # to be non-fatal.
-        log.exception('Failed to read from low-latency iDQ frame files')
-        return (channel, None)
-    else:
-        return (channel, float(idq_prob.max()))
+    if cache:
+        try:
+            idq_prob = TimeSeries.read(
+                cache, channel, start=start, end=end)
+            return (channel, float(idq_prob.max().value))
+        except (IndexError, RuntimeError):
+            # FIXME: Change from log.exception to log.warning until this fixed,
+            # because it's saturating Sentry.
+            log.warning('Failed to read from low-latency iDQ frame files')
+    # FIXME: figure out how to get access to low-latency frames outside
+    # of the cluster. Until we figure that out, actual I/O errors have
+    # to be non-fatal.
+    return (channel, None)
 
 
 def check_vector(cache, channel, start, end, bits, logic_type='all'):
@@ -286,30 +318,45 @@ def check_vector(cache, channel, start, end, bits, logic_type='all'):
      'H1:NO_CBC_HW_INJ': True,
      'H1:NO_BURST_HW_INJ': True,
      'H1:NO_DETCHAR_HW_INJ': True}
+
     """
     if logic_type not in ('any', 'all'):
         raise ValueError("logic_type must be either 'all' or 'any'.")
-    bitname = '{}:{}'
-    try:
-        statevector = StateVector.read(cache, channel, start=start, end=end,
-                                       bits=bits)
-    except IndexError:
-        # FIXME: figure out how to get access to low-latency frames outside
-        # of the cluster. Until we figure that out, actual I/O errors have
-        # to be non-fatal.
-        log.exception('Failed to read from low-latency frame files')
-        return {bitname.format(channel.split(':')[0], key):
-                None for key in bits if key is not None}
     else:
-        return {bitname.format(channel.split(':')[0], key):
-                bool(getattr(np, logic_type)(getattr(value, 'value')))
-                for key, value in statevector.get_bit_series().items()}
+        logic_map = {'any': np.any, 'all': np.all}
+    bitname = '{}:{}'
+    if cache:
+        try:
+            statevector = StateVector.read(cache, channel,
+                                           start=start, end=end, bits=bits)
+        except (IndexError, TypeError):
+            # FIXME: TypeError above is due to
+            # https://github.com/gwpy/gwpy/issues/1211
+            #
+            # FIXME: Change from log.exception to log.warning until this fixed,
+            # because it's saturating Sentry.
+            log.warning('Failed to read from low-latency frame files')
+        else:
+            # FIXME: In the playground environment, the Virgo state vector
+            # channel is stored as a float. Is this also the case in the
+            # production environment?
+            statevector = statevector.astype(np.uint32)
+            if len(statevector) > 0:  # statevector must not be empty
+                return {bitname.format(channel.split(':')[0], key):
+                        bool(logic_map[logic_type](
+                            value.value if len(value.value) > 0 else None))
+                        for key, value in statevector.get_bit_series().items()}
+    # FIXME: figure out how to get access to low-latency frames outside
+    # of the cluster. Until we figure that out, actual I/O errors have
+    # to be non-fatal.
+    return {bitname.format(channel.split(':')[0], key):
+            None for key in bits if key is not None}
 
 
 @app.task(shared=False)
 def check_vectors(event, graceid, start, end):
     """Perform data quality checks for an event and labels/logs results to
-    GraceDb.
+    GraceDB.
 
     Depending on the pipeline, a certain amount of time (specified in
     :obj:`~gwcelery.conf.check_vector_prepost`) is appended to either side of
@@ -321,15 +368,15 @@ def check_vectors(event, graceid, start, end):
     involved in the event. Then, the bits and channels specified in the
     configuration file (:obj:`~gwcelery.conf.llhoft_channels`) are checked.
     If an injection is found in the active detectors, 'INJ' is labeled to
-    GraceDb. If an injection is found in any detector, a message with the
-    injection found is logged to GraceDb. If no injections are found across
-    all detectors, this is logged to GraceDb.
+    GraceDB. If an injection is found in any detector, a message with the
+    injection found is logged to GraceDB. If no injections are found across
+    all detectors, this is logged to GraceDB.
 
     A similar task is performed for the DQ states described in the
     DMT-DQ_VECTOR, LIGO GDS-CALIB_STATE_VECTOR, and Virgo
     DQ_ANALYSIS_STATE_VECTOR. If no DQ issues are found in active detectors,
-    'DQOK' is labeled to GraceDb. Otherwise, 'DQV' is labeled. In all cases,
-    the DQ states of all the state vectors checked are logged to GraceDb.
+    'DQOK' is labeled to GraceDB. Otherwise, 'DQV' is labeled. In all cases,
+    the DQ states of all the state vectors checked are logged to GraceDB.
 
     This skips MDC events.
 
@@ -341,6 +388,12 @@ def check_vectors(event, graceid, start, end):
         GraceID of event to which to log.
     start, end : int or float
         GPS start and end times desired.
+
+    Returns
+    -------
+    event : dict
+        Details of the event, reflecting any labels that were added.
+
     """
     # Skip MDC events.
     if event.get('search') == 'MDC':
@@ -348,17 +401,6 @@ def check_vectors(event, graceid, start, end):
                  event['graceid'])
         return event
 
-    # Check the full template duration
-    pref_event = event.get('preferred_event')
-    if pref_event is not None:  # external events won't have preferred events
-        try:
-            template_dur = pref_event[
-                'extra_attributes']['SingleInspiral'][0]['template_duration']
-            start = start - template_dur
-        except KeyError:
-            # preferred event is a burst or has no template duration
-            pass
-
     # Create caches for all detectors
     instruments = event['instruments'].split(',')
     pipeline = event['pipeline']
@@ -370,6 +412,10 @@ def check_vectors(event, graceid, start, end):
     ifos = {key.split(':')[0] for key, val in
             app.conf['llhoft_channels'].items()}
     caches = {ifo: create_cache(ifo, start, end) for ifo in ifos}
+    bit_defs = {channel_type: Bits(channel=bitdef['channel'],
+                                   bits=bitdef['bits'])
+                for channel_type, bitdef
+                in app.conf['detchar_bit_definitions'].items()}
 
     # Examine injection and DQ states
     # Do not analyze DMT-DQ_VECTOR if pipeline uses gated h(t)
@@ -380,7 +426,7 @@ def check_vectors(event, graceid, start, end):
                              if k[3:] != 'DMT-DQ_VECTOR'}.items()
     for channel, bits in analysis_channels:
         states.update(check_vector(caches[channel.split(':')[0]], channel,
-                                   start, end, globals()[bits]))
+                                   start, end, bit_defs[bits]))
     # Pick out DQ and injection states, then filter for active detectors
     dq_states = {key: value for key, value in states.items()
                  if key.split('_')[-1] != 'INJ'}
@@ -396,42 +442,57 @@ def check_vectors(event, graceid, start, end):
                                channel, start, end)
                      for channel in app.conf['idq_channels'])
 
-    # Logging iDQ to GraceDb
+    # Logging iDQ to GraceDB
     if None not in idq_probs.values():
+        idq_probs_readable = {k: round(v, 3) for k, v in idq_probs.items()}
         if max(idq_probs.values()) >= app.conf['idq_pglitch_thresh']:
-            idq_probs_readable = {k: round(v, 3) for k, v in idq_probs.items()}
             idq_msg = ("iDQ glitch probability is high: "
                        "maximum p(glitch) is {}. ").format(
                 json.dumps(idq_probs_readable)[1:-1])
             # If iDQ p(glitch) is high and pipeline enabled, apply DQV
             if app.conf['idq_veto'][pipeline]:
+                gracedb.remove_label('DQOK', graceid)
                 gracedb.create_label('DQV', graceid)
+                # Add labels to return value to avoid querying GraceDB again.
+                event = dict(event, labels=event.get('labels', []) + ['DQV'])
+                try:
+                    event['labels'].remove('DQOK')
+                except ValueError:  # not in list
+                    pass
         else:
             idq_msg = ("iDQ glitch probabilities at both H1 and L1 "
                        "are good (below {} threshold). "
                        "Maximum p(glitch) is {}. ").format(
                            app.conf['idq_pglitch_thresh'],
-                           json.dumps(idq_probs)[1:-1])
+                           json.dumps(idq_probs_readable)[1:-1])
     else:
         idq_msg = "iDQ glitch probabilities unknown. "
-    gracedb.upload(
+    gracedb.upload.delay(
         None, None, graceid, idq_msg + prepost_msg, ['data_quality'])
 
-    # Labeling INJ to GraceDb
+    # Labeling INJ to GraceDB
     if False in active_inj_states.values():
         # Label 'INJ' if injection found in active IFOs
         gracedb.create_label('INJ', graceid)
+        # Add labels to return value to avoid querying GraceDB again.
+        event = dict(event, labels=event.get('labels', []) + ['INJ'])
     if False in inj_states.values():
-        # Write all found injections into GraceDb log
+        # Write all found injections into GraceDB log
         injs = [k for k, v in inj_states.items() if v is False]
         inj_fmt = "Injection found.\n{}\n"
         inj_msg = inj_fmt.format(
             generate_table('Injection bits', [], injs, []))
     elif all(inj_states.values()) and len(inj_states.values()) > 0:
         inj_msg = 'No HW injections found. '
+        gracedb.remove_label('INJ', graceid)
+        event = dict(event, labels=list(event.get('labels', [])))
+        try:
+            event['labels'].remove('INJ')
+        except ValueError:  # not in list
+            pass
     else:
         inj_msg = 'Injection state unknown. '
-    gracedb.upload(
+    gracedb.upload.delay(
         None, None, graceid, inj_msg + prepost_msg, ['data_quality'])
 
     # Determining overall_dq_active_state
@@ -456,15 +517,29 @@ def check_vectors(event, graceid, start, end):
     else:
         gate_msg = ''
 
-    # Labeling DQOK/DQV to GraceDb
-    gracedb.upload(
+    # Labeling DQOK/DQV to GraceDB
+    gracedb.upload.delay(
         None, None, graceid, msg + prepost_msg + gate_msg, ['data_quality'])
     if overall_dq_active_state is True:
         state = "pass"
+        gracedb.remove_label('DQV', graceid)
         gracedb.create_label('DQOK', graceid)
+        # Add labels to return value to avoid querying GraceDB again.
+        event = dict(event, labels=event.get('labels', []) + ['DQOK'])
+        try:
+            event['labels'].remove('DQV')
+        except ValueError:  # not in list
+            pass
     elif overall_dq_active_state is False:
         state = "fail"
+        gracedb.remove_label('DQOK', graceid)
         gracedb.create_label('DQV', graceid)
+        # Add labels to return value to avoid querying GraceDB again.
+        event = dict(event, labels=event.get('labels', []) + ['DQV'])
+        try:
+            event['labels'].remove('DQOK')
+        except ValueError:  # not in list
+            pass
     else:
         state = "unknown"
 
@@ -477,7 +552,7 @@ def check_vectors(event, graceid, start, end):
     file = dqr_json(json_state, state_summary)
     filename = 'gwcelerydetcharcheckvectors-{}.json'.format(graceid)
     message = "DQR-compatible json generated from check_vectors results"
-    gracedb.upload(
+    gracedb.upload.delay(
         json.dumps(file), filename, graceid, message)
 
     return event
