@@ -178,11 +178,11 @@ def handle_cbc_event(alert):
 
     """  # noqa: E501
     graceid = alert['uid']
+    pipeline = alert['object']['pipeline'].lower()
     priority = 0 if superevents.should_publish(alert['object']) else 1
 
     # em_bright and p_astro calculation
     if alert['alert_type'] == 'new':
-        pipeline = alert['object']['pipeline'].lower()
         instruments = superevents.get_instruments_in_ranking_statistic(
             alert['object'])
         extra_attributes = alert['object']['extra_attributes']
@@ -225,14 +225,41 @@ def handle_cbc_event(alert):
                 gracedb.create_label.si('PASTRO_READY', graceid)
             ).apply_async(priority=priority)
 
+        # Start BAYESTAR for PyCBC.
+        # PyCBC includes the PSD data in the initial upload,
+        # so we just download the coinc.xml file.
+        if pipeline == 'pycbc':
+            (
+                group(
+                    gracedb.download.s('coinc.xml', graceid)
+                )
+                |
+                bayestar.localize.s(graceid)
+                |
+                gracedb.upload.s(
+                    'bayestar.multiorder.fits', graceid,
+                    'sky localization complete', ['sky_loc', 'public']
+                )
+                |
+                gracedb.create_label.si('SKYMAP_READY', graceid)
+            ).apply_async(priority=priority)
+
     if alert['alert_type'] != 'log':
         return
 
     filename = alert['data']['filename']
 
-    if filename == 'psd.xml.gz':
+    # Start BAYESTAR for any pipeline *except* PyCBC.
+    # All pipelines but PyCBC upload the PSD in a separate file, psd.xml.gz.
+    # For those pipelines, BAYESTAR must download coinc.xml *and* psd.xml.gz.
+    #
+    # FIXME: The separate psd.xml.gz upload adds an extra couple seconds of
+    # latency due to the additional GraceDB transactions and ping times.
+    # If other pipelines were able to add the PSD to the initial upload,
+    # then we could cut down on the alert latency by a couple seconds.
+    if pipeline != 'pycbc' and filename == 'psd.xml.gz':
         (
-            ordered_group(
+            group(
                 gracedb.download.s('coinc.xml', graceid),
                 gracedb.download.s('psd.xml.gz', graceid)
             )
@@ -570,7 +597,10 @@ def preliminary_alert(event, superevent_id, annotation_prefix='',
             _proceed_if_no_advocate_action.s(superevent_id)
             |
             preliminary_initial_update_alert.s(
-                superevent_id, 'preliminary', labels=event['labels'])
+                superevent_id,
+                ('earlywarning' if event['search'] == 'EarlyWarning'
+                 else 'preliminary'),
+                labels=event['labels'])
         )
 
     canvas.apply_async(priority=priority)
@@ -645,7 +675,7 @@ def preliminary_initial_update_alert(filenames, superevent_id, alert_type,
         A list of the sky map, em_bright, and p_astro filenames.
     superevent_id : str
         The superevent ID.
-    alert_type : {'preliminary', 'initial', 'update'}
+    alert_type : {'earlywarning', 'preliminary', 'initial', 'update'}
         The alert type.
     labels : list
         A list of labels applied to superevent.
@@ -686,7 +716,7 @@ def preliminary_initial_update_alert(filenames, superevent_id, alert_type,
                     and f.endswith('.json'):
                 p_astro_filename = fv
 
-    if alert_type in ['preliminary', 'initial']:
+    if alert_type in {'earlywarning', 'preliminary', 'initial'}:
         if 'RAVEN_ALERT' in labels:
             circular_task = circulars.create_emcoinc_circular.si(superevent_id)
             circular_filename = '{}-emcoinc-circular.txt'.format(alert_type)
@@ -742,7 +772,8 @@ def preliminary_initial_update_alert(filenames, superevent_id, alert_type,
             |
             (
                 gracedb.create_label.si('GCN_PRELIM_SENT', superevent_id)
-                if alert_type == 'preliminary' else identity.si()
+                if alert_type in {'earlywarning', 'preliminary'}
+                else identity.si()
             ),
 
             circular_canvas,
