@@ -177,10 +177,13 @@ def process(payload):
     if should_publish(event_info):
         gracedb.create_label.delay('ADVREQ', sid)
         if is_complete(event_info):
-            gracedb.create_label.s(FROZEN_LABEL, sid).set(
-                queue='superevent',
-                countdown=app.conf['preliminary_alert_timeout']
-            ).delay()
+            if app.conf['preliminary_alert_timeout']:
+                gracedb.create_label.s(FROZEN_LABEL, sid).set(
+                    queue='superevent',
+                    countdown=app.conf['preliminary_alert_timeout']
+                ).delay()
+            else:  # fast path if no countdown
+                gracedb.create_label(FROZEN_LABEL, sid)
 
 
 def get_category(event):
@@ -410,7 +413,10 @@ def _should_publish(event):
     """
     group = event['group'].lower()
     trials_factor = app.conf['preliminary_alert_trials_factor'][group]
-    far_threshold = app.conf['preliminary_alert_far_threshold'][group]
+    if event.get('search') == 'EarlyWarning':
+        far_threshold = app.conf['early_warning_alert_far_threshold']
+    else:
+        far_threshold = app.conf['preliminary_alert_far_threshold'][group]
     far = trials_factor * event['far']
     raven_coincidence = ('RAVEN_ALERT' in event['labels'])
 
