@@ -4,21 +4,28 @@ import re
 
 from astropy.time import Time
 from flask import flash, jsonify, redirect, render_template, request, url_for
-from ligo.gracedb.rest import HTTPError as GraceDbHTTPError
+from flask import make_response
+from requests.exceptions import HTTPError
+import pkg_resources
 
 from . import app as celery_app
 from ._version import get_versions
 from .flask import app, cache
-from .tasks import first2years, gracedb, orchestrator
+from .tasks import first2years, gracedb, orchestrator, circulars, superevents
 
 
 @app.route('/')
 def index():
+    """Render main page."""
     return render_template(
-        'index.jinja2', conf=celery_app.conf, versions=get_versions())
+        'index.jinja2',
+        conf=celery_app.conf,
+        packages=pkg_resources.working_set,
+        versions=get_versions())
 
 
 def take_n(n, iterable):
+    """Take the first `n` items of a collection."""
     for i, item in enumerate(iterable):
         if i >= n:
             break
@@ -26,7 +33,7 @@ def take_n(n, iterable):
 
 
 # Regular expression for parsing query strings
-# that look like GraceDb superevent names.
+# that look like GraceDB superevent names.
 _typeahead_superevent_id_regex = re.compile(
     r'(?P<prefix>[MT]?)S?(?P<date>\d{0,6})(?P<suffix>[a-z]*)',
     re.IGNORECASE)
@@ -35,11 +42,11 @@ _typeahead_superevent_id_regex = re.compile(
 @app.route('/typeahead_superevent_id')
 @cache.cached(query_string=True)
 def typeahead_superevent_id():
-    """Search GraceDb for superevents by ID.
-
-    This involves some date parsing because GraceDb does not support directly
-    searching for superevents by ID substring."""
+    """Search GraceDB for superevents by ID.
 
+    This involves some date parsing because GraceDB does not support directly
+    searching for superevents by ID substring.
+    """
     max_results = 8  # maximum number of results to return
     batch_results = 32  # batch size for results from server
 
@@ -47,7 +54,7 @@ def typeahead_superevent_id():
     match = _typeahead_superevent_id_regex.fullmatch(term) if term else None
 
     if match:
-        # Determine GraceDb event category from regular expression.
+        # Determine GraceDB event category from regular expression.
         prefix = match['prefix'].upper() + 'S'
         category = {'T': 'test', 'M': 'MDC'}.get(
             match['prefix'].upper(), 'production')
@@ -68,7 +75,7 @@ def typeahead_superevent_id():
         date_end = date_start + datetime.timedelta(
             days=[36600, 3660, 366, 320, 32, 11, 1.1][date_partial_length])
 
-        # Determine GraceDb event suffix from regular expression.
+        # Determine GraceDB event suffix from regular expression.
         suffix = match['suffix'].lower()
     else:
         prefix = 'S'
@@ -79,11 +86,11 @@ def typeahead_superevent_id():
         date_partial_length = 0
         suffix = ''
 
-    # Query GraceDb.
+    # Query GraceDB.
     query = 'category: {} t_0: {} .. {}'.format(
         category, Time(date_start).gps, Time(date_end).gps)
-    response = gracedb.client.superevents(
-        query, orderby='superevent_id', count=batch_results)
+    response = gracedb.client.superevents.search(
+        query=query, sort='superevent_id', count=batch_results)
 
     # Filter superevent IDs that match the search term.
     regex = re.compile(r'{}{}\d{{{}}}{}[a-z]*'.format(
@@ -96,50 +103,37 @@ def typeahead_superevent_id():
     return jsonify(list(take_n(max_results, superevent_ids)))
 
 
-# Regular expression for parsing query strings
-# that look like GraceDb event names.
-_typeahead_event_id_regex = re.compile(
-    r'(?P<prefix>[GMT]?)(?P<number>\d*)',
-    re.IGNORECASE)
-
-
 @app.route('/typeahead_event_id')
 @cache.cached(query_string=True)
 def typeahead_event_id():
-    """Search GraceDb for events by ID."""
-
-    term = request.args.get('event_id')
-    match = _typeahead_event_id_regex.fullmatch(term) if term else None
-
-    if match:
-        # Determine GraceDb event category from regular expression.
-        prefix = match['prefix'].upper() or 'G'
-        number = int(match['number'] or '0')
-    else:
-        prefix = 'G'
-        number = 0
-
-    # Query GraceDb.
-    query = '{prefix}{number} {prefix}{number}0..{prefix}{number}9'.format(
-        prefix=prefix, number=number)
-    response = gracedb.client.events(query)
-    event_ids = reversed([event['graceid'] for event in response])
-
-    # Return only the first few matches.
-    return jsonify(list(event_ids))
+    """Search GraceDB for events by ID."""
+    superevent_id = request.args.get('superevent_id').strip()
+    query_terms = [f'superevent: {superevent_id}']
+    if superevent_id.startswith('T'):
+        query_terms.append('Test')
+    elif superevent_id.startswith('M'):
+        query_terms.append('MDC')
+    query = ' '.join(query_terms)
+    try:
+        results = gracedb.get_events(query)
+    except HTTPError:
+        results = []
+    results = [dict(r, snr=superevents.get_snr(r)) for r in results
+               if superevents.is_complete(r)]
+    return jsonify(list(reversed(sorted(results, key=superevents.keyfunc))))
 
 
 def _search_by_tag_and_filename(superevent_id, filename, extension, tag):
     try:
-        records = gracedb.client.logs(superevent_id).json()['log']
+        records = gracedb.get_log(superevent_id)
         return [
-            record['filename'] for record in records
-            if tag in record['tag_names']
+            '{},{}'.format(record['filename'], record['file_version'])
+            for record in records if tag in record['tag_names']
             and record['filename'].startswith(filename)
             and record['filename'].endswith(extension)]
-    except GraceDbHTTPError as e:
+    except HTTPError as e:
         # Ignore 404 errors from server
-        if e.status == 404:
+        if e.response.status_code == 404:
             return []
         else:
             raise
@@ -148,6 +142,7 @@ def _search_by_tag_and_filename(superevent_id, filename, extension, tag):
 @app.route('/typeahead_skymap_filename')
 @cache.cached(query_string=True)
 def typeahead_skymap_filename():
+    """Search for sky maps by filename."""
     return jsonify(_search_by_tag_and_filename(
         request.args.get('superevent_id') or '',
         request.args.get('filename') or '',
@@ -158,6 +153,7 @@ def typeahead_skymap_filename():
 @app.route('/typeahead_em_bright_filename')
 @cache.cached(query_string=True)
 def typeahead_em_bright_filename():
+    """Search em_bright files by filename."""
     return jsonify(_search_by_tag_and_filename(
         request.args.get('superevent_id') or '',
         request.args.get('filename') or '',
@@ -168,6 +164,7 @@ def typeahead_em_bright_filename():
 @app.route('/typeahead_p_astro_filename')
 @cache.cached(query_string=True)
 def typeahead_p_astro_filename():
+    """Search p_astro files by filename."""
     return jsonify(_search_by_tag_and_filename(
         request.args.get('superevent_id') or '',
         request.args.get('filename') or '',
@@ -177,6 +174,7 @@ def typeahead_p_astro_filename():
 
 @app.route('/send_preliminary_gcn', methods=['POST'])
 def send_preliminary_gcn():
+    """Handle submission of preliminary alert form."""
     keys = ('superevent_id', 'event_id')
     superevent_id, event_id, *_ = tuple(request.form.get(key) for key in keys)
     if superevent_id and event_id:
@@ -201,22 +199,50 @@ def send_preliminary_gcn():
     return redirect(url_for('index'))
 
 
-@app.route('/send_update_gcn', methods=['POST'])
-def send_update_gcn():
-    keys = ('superevent_id', 'skymap_filename',
-            'em_bright_filename', 'p_astro_filename')
-    superevent_id, *_ = args = tuple(request.form.get(key) for key in keys)
-    if all(args):
-        app.logger.info(
-            'Calling update_alert%r', args)
+@app.route('/change_prefered_event', methods=['POST'])
+def change_prefered_event():
+    """Handle submission of preliminary alert form."""
+    keys = ('superevent_id', 'event_id')
+    superevent_id, event_id, *_ = tuple(request.form.get(key) for key in keys)
+    if superevent_id and event_id:
         (
             gracedb.upload.s(
                 None, None, superevent_id,
-                'User {} queued a Preliminary alert through the dashboard.'
+                'User {} queued a prefered event change to {}.'
+                .format(request.remote_user or '(unknown)', event_id),
+                tags=['em_follow'])
+            |
+            gracedb.update_superevent.si(
+                superevent_id, preferred_event=event_id)
+            |
+            gracedb.get_event.si(event_id)
+            |
+            orchestrator.preliminary_alert.s(
+                superevent_id, initiate_voevent=False)
+        ).delay()
+        flash('Changed prefered event for {}.'.format(superevent_id),
+              'success')
+    else:
+        flash('No change performed. Please fill in all fields.', 'danger')
+    return redirect(url_for('index'))
+
+
+@app.route('/send_update_gcn', methods=['POST'])
+def send_update_gcn():
+    """Handle submission of update alert form."""
+    keys = ('superevent_id', 'skymap_filename',
+            'em_bright_filename', 'p_astro_filename')
+    superevent_id, *filenames = args = tuple(
+        request.form.get(key) for key in keys)
+    if all(args):
+        (
+            gracedb.upload.s(
+                None, None, superevent_id,
+                'User {} queued an Update alert through the dashboard.'
                 .format(request.remote_user or '(unknown)'),
                 tags=['em_follow'])
             |
-            orchestrator.update_alert.si(*args)
+            orchestrator.update_alert.si(filenames, superevent_id)
         ).delay()
         flash('Queued update alert for {}.'.format(superevent_id), 'success')
     else:
@@ -224,8 +250,27 @@ def send_update_gcn():
     return redirect(url_for('index'))
 
 
+@app.route('/create_update_gcn_circular', methods=['POST'])
+def create_update_gcn_circular():
+    """Handle submission of GCN Circular form."""
+    keys = ['sky_localization', 'em_bright', 'p_astro']
+    superevent_id = request.form.get('superevent_id')
+    updates = [key for key in keys if request.form.get(key)]
+    if superevent_id and updates:
+        response = make_response(circulars.create_update_circular(
+            superevent_id,
+            update_types=updates))
+        response.headers["content-type"] = "text/plain"
+        return response
+    else:
+        flash('No circular created. Please fill in superevent ID and at ' +
+              'least one update type.', 'danger')
+    return redirect(url_for('index'))
+
+
 @app.route('/send_mock_event', methods=['POST'])
 def send_mock_event():
+    """Handle submission of mock alert form."""
     first2years.upload_event.delay()
     flash('Queued a mock event.', 'success')
     return redirect(url_for('index'))
